@@ -37,16 +37,55 @@ def pathway_frame_from_file ( filename ,
 
 
 def create_dag_representation_df ( pathway_file:str = '../data/GROUPDEFINITIONS.gmt' ,
-                                   pcfile:str = '../data/PCLIST.txt', identifier:str='R-HSA'
+                                   pcfile:str = '../data/PCLIST.txt', identifier:str='R-HSA',
+                                   item_sep:str = ','
                                  ) :
     pc_list_file = pcfile
-    tree , ance , desc = parent_child_to_dag ( pc_list_file , identifier=identifier )
-    pdf = make_pathway_ancestor_data_frame ( ance )
+    tree , ance , desc = parent_child_to_dag ( pc_list_file , identifier = identifier )
     pdf_ = pathway_frame_from_file( pathway_file )
-    pdf.index = [v.replace(' ','') for v in  pdf.index.values]
-    pdf_.index= [v.replace(' ','') for v in pdf_.index.values]
-    dag_df = pd.concat([pdf.T,pdf_.T]).T
-    return ( dag_df , tree )
+    root = tree.get_root_id()
+    lost = set(tree.keys()) - set(pdf_.index.values.tolist())
+    for l in lost :
+        pdf_.loc[l] = ['NA','']
+    DF = []
+    for idx in pdf_.index.values :
+        ord_ascendants  = tree.search(  root_id         = idx                   ,
+                                        linktype        = 'ascendants'          ,
+                                        order           = 'depth'               )['path']
+        level = len ( ord_ascendants ) - 1
+        ord_descendants = tree.search(  root_id         = idx                   ,
+                                        linktype        = 'descendants'         ,
+                                        order           = 'depth'               )['path']
+        ord_ascendants  = [ a for a in ord_ascendants  if not ( a==idx ) ]
+        ord_descendants = [ d for d in ord_descendants if not ( d==idx ) ]
+        if len( ord_ascendants ) > 0 :
+            parent = ord_ascendants[0]
+        else:
+            parent = ''
+        dag_anscestors          = item_sep.join(  ord_ascendants )
+        dag_descendants         = item_sep.join( ord_descendants )
+        DF.append( [level,dag_anscestors,dag_descendants,parent , *pdf_.loc[idx].values ] )
+    ndag_df = pd.DataFrame( DF ,
+                  columns = [*['DAG,level','DAG,ancestors','DAG,descendants','DAG,parent'],*pdf_.columns.values],
+                  index   = pdf_.index.values )
+    return ( ndag_df , tree )
+
+
+def fill_in_full_hierarchy ( df:pd.DataFrame    = None ,
+                         gmtfile:str        = '/home/USER/data/Reactome/reactome_v71.gmt' ,
+                         pcfile :str        = '/home/USER/data/Reactome/NewestReactomeNodeRelations.txt',
+                         fields:list[float] = ['p-value'] ) -> pd.DataFrame :
+    dag_df , tree      = create_dag_representation_df ( pathway_file = gmtfile , pcfile = pcfile )
+    for field in fields :
+        dag_df.loc[:,field] = 1.
+    for idx in df.index.values :
+        dag_df.loc[ idx , fields ] = df.loc[ idx , fields ]
+    dag_df.loc[:,'parent'] = dag_df.loc[:,'DAG,parent']
+    for idx in dag_df.index.values :
+        if idx in set(df.index.values) :
+            dag_df.loc[idx,fields] = df.loc[idx,fields]
+    dag_df.loc[:,'name'] = dag_df.index.values
+    return ( dag_df )
 
 
 def HierarchalEnrichment (
@@ -64,10 +103,11 @@ def HierarchalEnrichment (
 
 
 def HierarchicalEnrichment (
-            analyte_df , dag_df , dag_level_label = 'DAG,l' ,
-            ancestors_id_label = 'aid' , id_name = None , threshold = 0.05 ,
-            p_label = 'C(Status),p', analyte_name_label = 'analytes' ,
-            item_delimiter = ',' , alexa_elim=False , alternative = 'two-sided'
+            analyte_df:pd.DataFrame , dag_df:pd.DataFrame , dag_level_label:str = 'DAG,l' ,
+            ancestors_id_label:str = 'aid' , id_name:str = None , threshold:float = 0.05 ,
+            p_label:str = 'C(Status),p', analyte_name_label:str = 'analytes' ,
+            item_delimiter:str = ',' , alexa_elim:bool=False , alternative:str = 'two-sided',
+            test_type:str = 'fisher', bNoMasking:bool=False, bOnlyMarkSignificant:bool=False
         ) :
     #
     # NEEDS AN ANALYTE SIGNIFICANCE FRAME:
@@ -76,6 +116,10 @@ def HierarchicalEnrichment (
     #     INCLUDING NODE ID, NODE ANALYTES FIELD (SEPERATED BY ITEM DELIMITER)
     #     INCLUDING ANCESTORS FIELD (SEPERATED BY ITEM DELIMITER)
     #     DAG LEVEL OF EACH NODE
+    from impetuous.special import unpack
+    all_annotated = set( [ w for w in unpack( [ str(v).split(item_delimiter)\
+                for v in dag_df.loc[:,analyte_name_label ].values.reshape(-1)\
+                        if not 'nan' == str(v).lower() ]) ])
     tolerance = threshold
     df = dag_df ; dag_depth = np.max( df[dag_level_label].values )
     AllAnalytes = set( analyte_df.index.values ) ; nidx = len( AllAnalytes )
@@ -83,7 +127,7 @@ def HierarchicalEnrichment (
     if len( AllAnalytes ) == len( SigAnalytes ) :
         print ( 'THIS STATISTICAL TEST WILL BE NONSENSE' )
         print ( 'TRY A DIFFERENT THRESHOLD' )
-    marked_analytes = {} ; used_analytes = {} ; node_sig = {}
+    marked_analytes = {} ; used_analytes = {} ; node_sig = {}; node_odds = {}
     for d in range( dag_depth, 0, -1 ) :
         # ROOT IS NOT INCLUDED
         filter_ = df [ dag_level_label ] == d
@@ -93,7 +137,7 @@ def HierarchicalEnrichment (
                 continue
             analytes_ = df.loc[node,analyte_name_label].replace('\n','').replace(' ','').split(item_delimiter)
             try :
-                group = analyte_df.loc[[a for a in analytes_ if a in AllAnalytes] ].dropna( axis=0, how='any', thresh=analyte_df.shape[1]/2 ).drop_duplicates()
+                group = analyte_df.loc[[a for a in analytes_ if a in AllAnalytes] ]
             except KeyError as e :
                 continue
             if node in marked_analytes :
@@ -102,10 +146,14 @@ def HierarchicalEnrichment (
             L_ = len( group ) ; str_analytes=','.join(group.index.values)
             if L_ > 0 :
                 used_analytes[node] = ','.join( group.index.values )
-                pv,odds = group_significance( group , AllAnalytes=AllAnalytes, SigAnalytes=SigAnalytes , tolerance = threshold , alternative=alternative )
-                node_sig[node] = pv ; marked_ = set( group.index.values )
+                pv , odds = group_significance( group ,
+				AllAnalytes = AllAnalytes, SigAnalytes = SigAnalytes , AllAnnotated=all_annotated ,
+				tolerance = threshold , alternative=alternative, TestType=test_type )
+                node_sig[node] = pv ; node_odds[node] = odds ; marked_ = set( group.index.values )
+                if bOnlyMarkSignificant : # NOT DEFAULT
+                    marked_ = marked_ & SigAnalytes
                 ancestors = df.loc[node,ancestors_id_label].replace('\n','').replace(' ','').split(item_delimiter)
-                if alexa_elim and pv > threshold : # USE ALEXAS ELIM ALGORITHM : IS NOT DEFAULT
+                if ( alexa_elim and pv > threshold ) or bNoMasking :  # USE ALEXAS ELIM ALGORITHM : IS NOT DEFAULT
                     continue
                 for u in ancestors :
                     if u in marked_analytes :
@@ -113,13 +161,13 @@ def HierarchicalEnrichment (
                         marked_analytes[u] = us | marked_
                     else :
                         marked_analytes[u] = marked_
-    df['Hierarchical,p'] = [ node_sig[idx] if idx in node_sig else 1. for idx in df.index.values ]
+    df['Hierarchical,p']	= [ node_sig[idx] if idx in node_sig else 1. for idx in df.index.values ]
+    df['Hierarchical,odds']	= [ node_odds[idx] if idx in node_sig else 1. for idx in df.index.values ]
     df['Included analytes,ids'] = [ used_analytes[idx] if idx in used_analytes else '' for idx in df.index.values ]
     df = df.dropna()
     return ( df )
 
 
-
 def hierarchy_matrix ( distance_matrix:np.array   = None ,
                        coordinates:np.array       = None ,
                        linkage_distances:np.array = None ) -> dict :
